<?php

namespace Drupal\commerce_paykeeper\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\Entity\Order;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsNotificationsInterface;


/**
 * Provides the Off-site Redirect payment Paykeeper gateway.
 *
 * @CommercePaymentGateway(
 *   id = "paykeeper_offsite_gateway",
 *   label = "Paykeeper offsite payment gateway",
 *   display_label = "Paykeeper",
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_paykeeper\PluginForm\OffsiteRedirect\PaykeeperpaymentForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "mastercard", "visa",
 *   },
 *   requires_billing_information = FALSE,
 * )
 */
class Paykeeperpayment extends OffsitePaymentGatewayBase implements SupportsNotificationsInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $config   = \Drupal::config('commerce_paykeeper.settings');
    $defaults = [];
    $defaults = $defaults + [
        'payment_url'     => $config->get('commerce_paykeeper.payment_url'),
        'secret_word'  => $config->get('commerce_paykeeper.secret_word'),
        'forced_discounts_check' => $config->get('commerce_paykeeper.forced_discounts_check'),
      ];
    return $defaults + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    //$form = parent::buildConfigurationForm($form, $form_state);

    $form['payment_url'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Payment URL'),
      '#default_value' => $this->configuration['payment_url'],
      '#type'          => 'textfield',
      '#required'      => TRUE,
    ];

    $form['secret_word'] = [
      '#title'         => $this->t('Secret word'),
      '#default_value' => $this->configuration['secret_word'],
      '#type'          => 'textfield',
      '#required'      => TRUE,
    ];

    $form['forced_discounts_check'] = [
      '#title'         => $this->t('Forced discounts check'),
      '#default_value' => $this->configuration['forced_discounts_check'],
      '#type'          => 'checkbox',
      '#required'      => TRUE,
    ];
    return $form;
  }



  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);

    $values = $form_state->getValue($form['#parents']);
    $required_fields = [
      'payment_url',
      'secret_word',
      'forced_discounts_check',
    ];
    foreach ($required_fields as $key) {
      if (empty($values[$key])) {
        $this->messenger()->addError($this->t('Paykeeper service is not configured for use. Please contact an administrator to resolve this issue.'));
        return FALSE;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
  {
    //parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors())
    {
      $values                             = $form_state->getValue($form['#parents']);
      $this->configuration['payment_url']     = $values['payment_url'];
      $this->configuration['secret_word']  = $values['secret_word'];
      $this->configuration['forced_discounts_check'] = $values['forced_discounts_check'];

    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {

  }

  /**
   * {@inheritdoc}
   */
  public function onCancel(OrderInterface $order, Request $request) {
    $this->messenger()->addMessage($this->t('You have canceled checkout at @gateway but may resume the checkout process here when you are ready.', [
      '@gateway' => $this->getDisplayLabel(),
    ]));
    throw new PaymentGatewayException('Payment cancelled by user');
  }

  /**
   * Getting configuration.
   *
   * @return array
   *   Configuration array.
   */
  public function getStoreDataConfiguration() {
    return $store_data = [
      'payment_url'             => isset($this->configuration['payment_url']) ? $this->configuration['payment_url'] : '',
      'private_key'             => isset($this->configuration['private_key']) ? $this->configuration['private_key'] : '',
      'forced_discounts_check'  => isset($this->configuration['forced_discounts_check']) ? $this->configuration['forced_discounts_check'] : '',
    ];
    return $store_data;
  }


  /**
   * Currency default validation.
   *
   * @param string $transaction_currency
   *   Currency from Liqpay.
   * @param string $order_currency
   *   Currency from order.
   *
   * @return bool
   *   TRUE on successful validation FALSE otherwise.
   */
  protected function validateCurrency($transaction_currency, $order_currency) {
    if ($transaction_currency != $order_currency) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Amount default validation.
   *
   * @param float $transaction_amount
   *   Amount from Liqpay.
   * @param float $order_amount
   *   Amount from order.
   *
   * @return bool
   *   TRUE on successful validation FALSE otherwise.
   */
  protected function validateAmount($transaction_amount, $order_amount) {
    if ($transaction_amount != $order_amount) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * IPN callback.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Symfony\Component\HttpFoundation\Response|null
   *   The response, or NULL to return an empty HTTP 200 response
   */
  public function onNotify(Request $request) {
      $response = new Response();
    if ($request->getMethod()=='GET')
    {
        $response->setContent("Некорректные входные данные!");
        return $response;
    }
    if ($request->getMethod()=='POST')
    {
      try
      {
        $order = Order::load($request->request->get('orderid'));
      }
      catch(Exception $e)
      {
        $response->setContent($e);
        return $response;
      }
      if(is_null($order))
      {
        $response->setContent('Order not exist!!!');
        return $response;
      }
      $secret_seed = $this->configuration['secret_word'];
      $id = $request->request->get('id');
      $sum = $request->request->get('sum');
      $clientid = $request->request->get('clientid');
      $orderid = $request->request->get('orderid');
      $key =  $request->request->get('key');
      if ($key != md5 ($id . sprintf ("%.2lf", $sum).$clientid.$orderid.$secret_seed))
      {
        $response->setContent('Error! Hash mismatch');
        return $response;
      }
      $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
      $payment         = $payment_storage->create([
        'state'           => 'comleted',
        'amount'          => $order->getTotalPrice(),
        'payment_gateway' => $this->parentEntity->id(),
        'order_id'        => $request->request->get('orderid'),
        'remote_id'       => $request->request->get('orderid'),
        'remote_state'    => 'payed',
        'authorized'      => time(),
        'completed'      => time(),
      ]);
      $payment->save();
      $response->setContent('OK '. md5 ($id.$secret_seed));
      return $response;
    }

  }

}
